==> lms-0.0.x-dev.tar.gz: lms/opigno_learning_path/src/Controller/TrainingController.php <==
<?php

declare(strict_types=1);

namespace Drupal\opigno_learning_path\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\opigno_learning_path\Entity\Bundle\LearningPath;
use Drupal\opigno_learning_path\TrainingManager;
use Drupal\opigno_module\Entity\OpignoActivity;
use Drupal\opigno_module\Entity\OpignoModule;
use Drupal\opigno_module\Entity\UserModuleStatus;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * The training controller.
 */
final class TrainingController extends ControllerBase {

  use TrainingControllerTrait;

  /**
   * The training manager.
   */
  protected TrainingManager $trainingManager;

  /**
   * {@inheritdoc}
   */
  public function __construct(TrainingManager $trainingManager) {
    $this->trainingManager = $trainingManager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('opigno_learning_path.training_manager')
    );
  }

  /**
   * Start the learning path.
   *
   * This page will redirect the user to the first learning path content.
   */
  public function start(LearningPath $group) {
    try {
      $parameters = $this->trainingManager->getCurrentActivityParameters($group, $this->currentUser());
      // Training finished.
      if (\count($parameters) === 0) {
        $this->messenger()->addStatus($this->t('Training finished.'));
        return $this->redirect('entity.group.canonical', [
          'group' => $group->id(),
        ]);
      }
      [
        'lp_status' => $lp_status,
        'module_status' => $module_status,
        'activity_id' => $activity_id,
      ] = $parameters;
      $module_status->save();
    }
    catch (\Exception $e) {
      $url = $this->handleError($group, $e);
      return $this->redirect($url->getRouteName(), $url->getRouteParameters());
    }

    return $this->redirect('opigno_module.group.answer_form', [
      'group' => $lp_status->get('gid')->target_id,
      'opigno_module' => $module_status->get('module')->target_id,
      'opigno_activity' => $activity_id,
    ]);
  }

  /**
   * Specific activity callback.
   */
  public function activity(
    LearningPath $group,
    OpignoModule $opigno_module,
    OpignoActivity $opigno_activity
  ) {
    try {
      [
        'lp_status' => $lp_status,
        'module_status' => $module_status,
        'activity_id' => $activity_id,
      ] = $this->trainingManager->getCurrentActivityParameters($group, $this->currentUser());
    }
    catch (\Exception $e) {
      $url = $this->handleError($group, $e);
      return $this->redirect($url->getRouteName(), $url->getRouteParameters());
    }

    // Check if the student can take this activity. If not - redirect.
    $same_activity = $activity_id === $opigno_activity->id();
    $same_module = $module_status->get('module')->target_id === $opigno_module->id();

    // We are exactly where we are supposed to be (back / next button nav).
    if ($same_activity && $same_module) {
      return $this->getAnswerForm($module_status);
    }

    // Check if the user can access the requested activity otherwise
    // (browser back button or sidebar menu navigation).
    $access = FALSE;
    $reason = NULL;
    $answered = FALSE;

    if (!$same_module) {
      $requested_module_status = $this->trainingManager->loadModuleStatus($lp_status->id(), $opigno_module->id());
    }
    else {
      $requested_module_status = $module_status;
    }

    if ($requested_module_status !== NULL) {
      $answer = $this->trainingManager->loadAnswer($requested_module_status, $opigno_activity);
      if ($answer !== NULL) {
        $answered = TRUE;
      }
    }

    // Allow arbitrary activity case.
    if ($group->allowArbitraryActivity($answered)) {
      $access = TRUE;
    }

    // Allow going back in a module case.
    elseif (
      $same_module &&
      $opigno_activity->id() === $requested_module_status->getPreviousActivityId()
    ) {
      if ($opigno_module->getBackwardsNavigation()) {
        $access = TRUE;
      }
      else {
        $reason = $this->t('This module does not allow backwards navigation.');
      }
    }

    if (!$access) {
      if ($reason === NULL) {
        $reason = $this->t('This training does not allow free navigation.');
      }
      $this->messenger()->addWarning($reason);
      return $this->redirect('opigno_module.group.answer_form', [
        'group' => $group->id(),
        'opigno_module' => $module_status->get('module')->target_id,
        'opigno_activity' => $activity_id,
      ]);
    }

    // Different module - initialize the module status if not previously
    // taken with the requested activity and update LPStatus information.
    if (!$same_module) {
      if ($requested_module_status === NULL) {
        try {
          $requested_module_status = $this->trainingManager->initializeModule($lp_status, $opigno_module, $opigno_activity);
        }
        catch (\Exception $e) {
          $url = $this->handleError($group, $e);
          return $this->redirect($url->getRouteName(), $url->getRouteParameters());
        }
      }
      else {
        $requested_module_status->set('current_activity', $opigno_activity);
      }
      $requested_module_status->save();
      $lp_status->set('current_module_status', $requested_module_status);
      $lp_status->save();
    }

    // Different activity - all we need to do is update the current
    // module status activity.
    elseif (!$same_activity) {
      $requested_module_status->set('current_activity', $opigno_activity);
      $requested_module_status->save();
    }

    return $this->getAnswerForm($requested_module_status);
  }

  /**
   * Answer form getter method.
   *
   * @return mixed
   *   Renderable array or redirect response.
   */
  private function getAnswerForm(UserModuleStatus $module_status) {
    $module = $module_status->get('module')->entity;

    // Check if the module is open.
    // @todo Design a more elegant solution to be able to lock trainings,
    // preferably on the Learning Path level.
    $availability = $module->checkModuleAvailability();
    if ($availability['open'] === FALSE) {
      $this->messenger()->addWarning($availability['message']);
      return $this->redirect('entity.group.canonical', [
        'group' => $module_status->get('lp_status')->entity->get('gid')->target_id,
      ]);
    }

    // Activity.
    $build['module_title'] = [
      '#type' => 'html_tag',
      '#tag' => 'h2',
      '#value' => $module->label(),
      '#attributes' => [
        'class' => [
          'module-title',
        ],
      ],
    ];
    $activity = $module_status->get('current_activity')->entity;
    $build['activity_build'] = $this->entityTypeManager()->getViewBuilder('opigno_activity')->view($activity, 'activity');

    // Answer form.
    $answer = $this->trainingManager->loadAnswer($module_status, $activity);
    if ($answer === NULL) {
      $answer = $this->trainingManager->createAnswer($module_status, $activity);
    }
    $build['form'] = $this->entityFormBuilder()->getForm($answer, 'default', [
      'group' => $module_status->get('lp_status')->entity->getTraining(),
      'opigno_module' => $module,
      'opigno_activity' => $activity,
    ]);

    return $build;
  }

}

==> lms-0.2.x-dev.tar.gz: lms/opigno_learning_path/src/Controller/TrainingController.php <==
<?php

declare(strict_types=1);

namespace Drupal\opigno_learning_path\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Render\Markup;
use Drupal\opigno_learning_path\Entity\Bundle\LearningPath;
use Drupal\opigno_learning_path\TrainingManager;
use Drupal\opigno_module\Entity\OpignoActivity;
use Drupal\opigno_module\Entity\OpignoModule;
use Drupal\opigno_module\Entity\UserModuleStatus;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * The training controller.
 */
final class TrainingController extends ControllerBase {

  use TrainingControllerTrait;

  /**
   * The training manager.
   */
  protected TrainingManager $trainingManager;

  /**
   * {@inheritdoc}
   */
  public function __construct(TrainingManager $trainingManager) {
    $this->trainingManager = $trainingManager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('opigno_learning_path.training_manager')
    );
  }

  /**
   * Start the learning path.
   *
   * This page will redirect the user to the first learning path content.
   */
  public function start(LearningPath $group) {
    try {
      $parameters = $this->trainingManager->getCurrentActivityParameters($group, $this->currentUser());
      // Training finished.
      if (\count($parameters) === 0) {
        $this->messenger()->addStatus($this->t('Training finished.'));
        return $this->redirect('entity.group.canonical', [
          'group' => $group->id(),
        ]);
      }
      [
        'lp_status' => $lp_status,
        'module_status' => $module_status,
        'activity_id' => $activity_id,
      ] = $parameters;
      $module_status->save();
    }
    catch (\Exception $e) {
      $url = $this->handleError($group, $e);
      return $this->redirect($url->getRouteName(), $url->getRouteParameters());
    }

    return $this->redirect('opigno_module.group.answer_form', [
      'group' => $lp_status->get('gid')->target_id,
      'opigno_module' => $module_status->get('module')->target_id,
      'opigno_activity' => $activity_id,
    ]);
  }

  /**
   * Specific activity callback.
   */
  public function activity(
    LearningPath $group,
    OpignoModule $opigno_module,
    OpignoActivity $opigno_activity
  ) {
    try {
      [
        'lp_status' => $lp_status,
        'module_status' => $module_status,
        'activity_id' => $activity_id,
      ] = $this->trainingManager->getCurrentActivityParameters($group, $this->currentUser());
    }
    catch (\Exception $e) {
      $url = $this->handleError($group, $e);
      return $this->redirect($url->getRouteName(), $url->getRouteParameters());
    }

    // Check if the student can take this activity. If not - redirect.
    $same_activity = $activity_id === $opigno_activity->id();
    $same_module = $module_status->get('module')->target_id === $opigno_module->id();

    // We are exactly where we are supposed to be (back / next button nav).
    if ($same_activity && $same_module) {
      return $this->getAnswerForm($module_status);
    }

    // Check if the user can access the requested activity otherwise
    // (browser back button or sidebar menu navigation).
    $access = FALSE;
    $reason = NULL;
    $answered = FALSE;

    if (!$same_module) {
      $requested_module_status = $this->trainingManager->loadModuleStatus($lp_status->id(), $opigno_module->id());
    }
    else {
      $requested_module_status = $module_status;
    }

    if ($requested_module_status !== NULL) {
      $answer = $this->trainingManager->loadAnswer($requested_module_status, $opigno_activity);
      if ($answer !== NULL) {
        $answered = TRUE;
      }
    }

    // Allow arbitrary activity case.
    if ($group->allowArbitraryActivity($answered)) {
      $access = TRUE;
    }

    // Allow going back in a module case.
    elseif (
      $same_module &&
      $opigno_activity->id() === $requested_module_status->getPreviousActivityId()
    ) {
      if ($opigno_module->getBackwardsNavigation()) {
        $access = TRUE;
      }
      else {
        $reason = $this->t('This module does not allow backwards navigation.');
      }
    }

    if (!$access) {
      if ($reason === NULL) {
        $reason = $this->t('This training does not allow free navigation.');
      }
      $this->messenger()->addWarning($reason);
      return $this->redirect('opigno_module.group.answer_form', [
        'group' => $group->id(),
        'opigno_module' => $module_status->get('module')->target_id,
        'opigno_activity' => $activity_id,
      ]);
    }

    // Different module - initialize the module status if not previously
    // taken with the requested activity and update LPStatus information.
    if (!$same_module) {
      if ($requested_module_status === NULL) {
        try {
          $requested_module_status = $this->trainingManager->initializeModule($lp_status, $opigno_module, $opigno_activity);
        }
        catch (\Exception $e) {
          $url = $this->handleError($group, $e);
          return $this->redirect($url->getRouteName(), $url->getRouteParameters());
        }
      }
      else {
        $requested_module_status->set('current_activity', $opigno_activity);
      }
      $requested_module_status->save();
      $lp_status->set('current_module_status', $requested_module_status);
      $lp_status->save();
    }

    // Different activity - all we need to do is update the current
    // module status activity.
    elseif (!$same_activity) {
      $requested_module_status->set('current_activity', $opigno_activity);
      $requested_module_status->save();
    }

    return $this->getAnswerForm($requested_module_status);
  }

  /**
   * Answer form getter method.
   *
   * @return mixed
   *   Renderable array or redirect response.
   */
  private function getAnswerForm(UserModuleStatus $module_status) {
    $module = $module_status->get('module')->entity;

    // Check if the module is open.
    // @todo Design a more elegant solution to be able to lock trainings,
    // preferably on the Learning Path level.
    $availability = $module->checkModuleAvailability();
    if ($availability['open'] === FALSE) {
      $this->messenger()->addWarning(Markup::create($availability['message']));
      return $this->redirect('entity.group.canonical', [
        'group' => $module_status->get('lp_status')->entity->get('gid')->target_id,
      ]);
    }

    // Activity.
    $build['module_title'] = [
      '#type' => 'html_tag',
      '#tag' => 'h2',
      '#value' => $module->label(),
      '#attributes' => [
        'class' => [
          'module-title',
        ],
      ],
    ];
    $activity = $module_status->get('current_activity')->entity;
    $build['activity_build'] = $this->entityTypeManager()->getViewBuilder('opigno_activity')->view($activity, 'activity');

    // Answer form.
    $answer = $this->trainingManager->loadAnswer($module_status, $activity);
    if ($answer === NULL) {
      $answer = $this->trainingManager->createAnswer($module_status, $activity);
    }
    $build['form'] = $this->entityFormBuilder()->getForm($answer, 'default', [
      'group' => $module_status->get('lp_status')->entity->getTraining(),
      'opigno_module' => $module,
      'opigno_activity' => $activity,
    ]);

    return $build;
  }

}
